"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _path = _interopRequireDefault(require("path"));
var _child_process = require("child_process");
var _FileUtilities = _interopRequireDefault(require("../../utils/FileUtilities"));
var _hash = require("../../utils/hash");
var _logger = require("../../../build/utils/logger");
var _PathUtilities = _interopRequireDefault(require("../../utils/PathUtilities"));
class SonarQube {
  /**
  * @function getSonarHostUrl - get sonar host url  from property file
  * @returns {string}
  */

  static getSonarHostUrl() {
    var {
      metricServerHost
    } = _FileUtilities.default.getLintConfiguration();
    return metricServerHost;
  }

  /**
   * @function getProjectName - get  project name for sonarQube project
   * @param {Object} cliArgument -  Object represent cli Argument
   * @returns {string}
   */

  static getProjectName(cliArgument) {
    var date = new Date();
    var project_name;
    var timeSuffix = `${date.getUTCDate()}_${date.getHours()}_${`UTC`}`;
    const {
      dir = null,
      cmdExecuted,
      env,
      developerName = undefined
    } = cliArgument;
    const {
      branchDiffPath
    } = _FileUtilities.default.getLintConfiguration();
    const default_project = this.getProjectNameFrmGit(developerName);
    if (dir && env == "dev") {
      return `${default_project}_Dev-report`;
    }
    if (checkIfFileExists(branchDiffPath) && env == "ci") {
      switch (cmdExecuted) {
        case "lint-ci":
          {
            project_name = `${default_project}_CI-pipeline`;
            break;
          }
        case "exemption-report":
          {
            project_name = `${default_project}_Exemption-report`;
            break;
          }
        case "dev-ci":
          {
            project_name = `${default_project}_Dev-pipeline`;
            break;
          }
        case "precommit":
          {
            project_name = `${default_project}_Precommit`;
            break;
          }
        default:
          {
            project_name = `${default_project}_Dev-report`;
            break;
          }
      }
    }
    // var project_name =  dir ? `${default_project}_Dev-pipeline` : checkIfFileExists(branchDiffPath) && cmdExecuted == 'exemption-report' ?`${default_project}_Exemption-report` : checkIfFileExists(branchDiffPath) && cmdExecuted == 'lint-ci'   ? `${default_project}_CI-pipeline` : `${default_project}_Dev-pipeline`;
    return project_name;
  }

  /**
   * @function getProjectNameForExemption - get project name for ExemptionFiles
   * @returns string
   */

  static getProjectNameForExemption(cliParams) {
    const {
      developerName = undefined
    } = cliParams;
    const default_project = this.getProjectNameFrmGit(developerName);
    return `${default_project}_Exemption-report`;
  }

  /**
   * @function getSonarProperties -  collect all sonar properties
   * @param {Object} cliObjects -  Object represent cli Argument
   * @returns {Array<Object>}
   */

  static getSonarProperties(cliObjects) {
    const {
      env,
      dir
    } = cliObjects;
    const {
      branchDiffPath
    } = _FileUtilities.default.getLintConfiguration();
    const {
      JsFiles,
      featureFiles
    } = _FileUtilities.default.getFiles(env, dir, branchDiffPath);
    const sonarqube_host_url = this.getSonarHostUrl();
    const sonar_scanner_path = _PathUtilities.default.getSonarJarPath();
    const metric_token = this.getCredentials();
    var project_name = this.getProjectName(cliObjects);
    var resultFilesPath = featureFiles;
    var source_directories = JsFiles;
    const tsConfigurationPath = this.getTsConfigurationPath();
    const command = ["-jar", sonar_scanner_path, `-Dsonar.host.url=${sonarqube_host_url}`, `-Dsonar.projectKey=${project_name}`, `-Dsonar.login=${metric_token}`, `-Dsonar.sources=${source_directories}`, `-Dsonar.tests=${resultFilesPath}`, "-Dsonar.language=js", `-Dsonar.working.directory=${this.getSonarArtifactsDir()}`, `-Dsonar.eslint.reportPaths=${_PathUtilities.default.getLintReportPath()}`, "-Dsonar.javascript.lcov.reportPaths=coverage/lcov-report/lcov.info"];
    _FileUtilities.default.checkIfFileExists(tsConfigurationPath) && command.push(`-Dsonar.typescript.tsconfigPath=${tsConfigurationPath}`);
    return command;
  }

  /**
   * @function getProjectNameFrmGit - get project from git
   * @returns {string}
   */

  static getProjectNameFrmGit(developerName) {
    let $repoName = this.getRepoName();
    let $branchName = this.getBranchName();
    let $username = this.getGitUsername(developerName);
    return $repoName && $branchName && $username ? `${$repoName}_${$branchName}_${$username}` : "desk_default_projects";
  }

  /**
   * @function getGitUsername - get username from git
   * @returns {string}
   */

  static getGitUsername(developerName) {
    try {
      if (developerName) {
        return developerName.split("@")[0];
      }
      const userName = this.getSpawnOutput("git", ["config", "user.email"]);
      return userName.pop().split("@")[0];
    } catch (error) {
      _logger.Logger.log(_logger.Logger.INFO_TYPE, `make sure that you have propery setup git`);
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
      process.exit(0);
    }
  }

  /**
   * @function getRepoName - get repo name from git
   * @returns {string}
   */

  static getRepoName() {
    try {
      const repoName = this.getSpawnOutput("git", ["remote", "get-url", "origin"]);
      return repoName.pop().split("/").pop().split(".")[0];
    } catch (error) {
      _logger.Logger.log(_logger.Logger.INFO_TYPE, `make sure that you have propery setup git`);
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
      process.exit(0);
    }
  }

  /**
   * @function getBranchName - get branch name from git
   * @returns {string}
   */

  static getBranchName() {
    // git rev-parse --abbrev-ref HEAD
    try {
      var branchName = this.getSpawnOutput("git", ["rev-parse", "--abbrev-ref", "HEAD"]); // to get current branch
      return branchName.shift().trim();
    } catch (error) {
      _logger.Logger.log(_logger.Logger.INFO_TYPE, `make sure that you have propery setup git`);
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
      process.exit(0);
    }
  }

  /**
   * @function getSpawnOutput -  run execution in spawn method
   * @param {string} command - prefix command
   * @param {Array<string>} params - command 
   * @returns {string | any}
   */

  static getSpawnOutput(command, params = []) {
    const demon_process = (0, _child_process.spawnSync)(command, params, {
      shell: true
    });
    return demon_process.stdout.toString().split("\n").filter(Boolean);
  }

  /**
   * @function getCredentials -  get credentials for sonarQube
   * @returns {Object}
   */

  static getCredentials() {
    const {
      metric_token
    } = _FileUtilities.default.getLintConfiguration();
    return (0, _hash.decrypt)(metric_token, 7);
  }

  /**
   * @function getSonarArtifactsDir - get artifacts path for sonarQube
   * @returns {string}
   */

  static getSonarArtifactsDir() {
    return _path.default.resolve(process.cwd(), "lint-report", ".scannerwork");
  }
  static getTsConfigurationPath() {
    const {
      tsConfigurationPath = undefined
    } = _FileUtilities.default.getLintConfiguration();
    return tsConfigurationPath && _FileUtilities.default.checkIfFileExists(tsConfigurationPath) ? tsConfigurationPath : undefined;
  }

  /**
   * @function getSummaryOfLint -  get summary of lint process 
   * @param {Object} cliParams - Object represent cli Argument 
   * @returns {Object}
   */

  static getSummaryOfLint(cliParams) {
    var $projectName = this.getProjectName(cliParams);
    const {
      metricServerHost
    } = _FileUtilities.default.getLintConfiguration();
    const url = global.analytics.sonarQubeStatus === true ? `${metricServerHost}/dashboard?id=${$projectName}` : metricServerHost;
    return {
      web_url: url,
      error: global.analytics.status == "SUCCESS" ? {} : {
        code: "Issue in code detected",
        category: "code",
        message: `Click the link to view the detailed report - ${url}`
      },
      results: [{
        jsonReportPath: _PathUtilities.default.getLintReportPath().toString(),
        projectName: $projectName
      }]
    };
  }

  /**
   * @function getLintStatus - get status of lint based on report
   * @param {Boolean} impactBased - argument from cli to decide to lint based on impact or not
   * @returns {Object}
   */

  static getLintStatus(impactBased) {
    if (impactBased == false) {
      return {
        status: "SUCCESS",
        message: "No issues are found, your code adheres to the required standards."
      };
    }
    return {
      status: global.analytics.status,
      message: global.analytics.message
    };
  }

  /**
   * @function writeSummaryOfLint - write a summary of lint report ans it's status
   * @param {Object} cliParams -  Object represent cli Argument 
   * @returns {void}
   */

  static writeSummaryOfLint(cliParams) {
    const {
      impactBased
    } = _FileUtilities.default.getLintConfiguration();
    _FileUtilities.default.writeFileContents(_path.default.resolve(process.cwd(), "lint-report", "lint-summary.json"), JSON.stringify({
      ...this.getLintStatus(impactBased),
      ...this.getSummaryOfLint(cliParams)
    }));
  }
}
var _default = exports.default = SonarQube;