"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getBranchName = getBranchName;
exports.getCredentials = getCredentials;
exports.getGitUsername = getGitUsername;
exports.getProjectName = getProjectName;
exports.getProjectNameForExemption = getProjectNameForExemption;
exports.getProjectNameFrmGit = getProjectNameFrmGit;
exports.getRepoName = getRepoName;
exports.getSonarArtifactsDir = getSonarArtifactsDir;
exports.getSonarHostUrl = getSonarHostUrl;
exports.getSonarProperties = getSonarProperties;
exports.getSpawnOutput = getSpawnOutput;
var _rootPath = require("../../utils/rootPath");
var _path = _interopRequireDefault(require("path"));
var _child_process = require("child_process");
var _getFileUtils = require("../eslint/getFileUtils");
var _fileUtils = require("../../utils/fileUtils");
var _hash = require("../../utils/hash");
var _logger = require("../../../build/utils/logger");
/**
 * @function getSonarHostUrl - get sonar host url  from property file
 * @returns {string}
 */

function getSonarHostUrl() {
  var {
    metricServerHost
  } = (0, _fileUtils.getLintConfiguration)();
  return metricServerHost;
}

/**
 * @function getProjectName - get  project name for sonarQube project
 * @param {Object} cliArgument -  Object represent cli Argument
 * @returns {string}
 */

function getProjectName(cliArgument) {
  var date = new Date();
  var project_name;
  var timeSuffix = `${date.getUTCDate()}_${date.getHours()}_${`UTC`}`;
  const {
    dir = null,
    cmdExecuted,
    env,
    developerName = undefined
  } = cliArgument;
  const {
    branchDiffPath
  } = (0, _fileUtils.getLintConfiguration)();
  const default_project = getProjectNameFrmGit(developerName);
  if (dir && env == "dev") {
    return `${default_project}_Dev-report`;
  }
  if ((0, _fileUtils.checkIfFileExists)(branchDiffPath) && env == "ci") {
    switch (cmdExecuted) {
      case "lint-ci":
        {
          project_name = `${default_project}_CI-pipeline`;
          break;
        }
      case "exemption-report":
        {
          project_name = `${default_project}_Exemption-report`;
          break;
        }
      case "dev-ci":
        {
          project_name = `${default_project}_Dev-pipeline`;
          break;
        }
      case "precommit":
        {
          project_name = `${default_project}_Precommit`;
          break;
        }
      default:
        {
          project_name = `${default_project}_Dev-report`;
          break;
        }
    }
  }
  // var project_name =  dir ? `${default_project}_Dev-pipeline` : checkIfFileExists(branchDiffPath) && cmdExecuted == 'exemption-report' ?`${default_project}_Exemption-report` : checkIfFileExists(branchDiffPath) && cmdExecuted == 'lint-ci'   ? `${default_project}_CI-pipeline` : `${default_project}_Dev-pipeline`;
  return project_name;
}

/**
 * @function getProjectNameForExemption - get project name for ExemptionFiles
 * @returns string
 */

function getProjectNameForExemption(cliParams) {
  const {
    developerName = undefined
  } = cliParams;
  const default_project = getProjectNameFrmGit(developerName);
  return `${default_project}_Exemption-report`;
}

/**
 * @function getSonarProperties -  collect all sonar properties
 * @param {Object} cliObjects -  Object represent cli Argument
 * @returns {Array<Object>}
 */

function getSonarProperties(cliObjects) {
  const {
    env,
    dir
  } = cliObjects;
  const {
    branchDiffPath
  } = (0, _fileUtils.getLintConfiguration)();
  const {
    JsFiles,
    featureFiles
  } = (0, _getFileUtils.getFiles)(env, dir, branchDiffPath);
  const sonarqube_host_url = getSonarHostUrl();
  const sonar_scanner_path = (0, _rootPath.getSonarJarPath)();
  const metric_token = getCredentials();
  var project_name = getProjectName(cliObjects);
  var resultFilesPath = featureFiles;
  var source_directories = JsFiles;
  const tsConfigurationPath = getTsConfigurationPath();
  const command = ["-jar", sonar_scanner_path, `-Dsonar.host.url=${sonarqube_host_url}`, `-Dsonar.projectKey=${project_name}`, `-Dsonar.login=${metric_token}`, `-Dsonar.sources=${source_directories}`, `-Dsonar.tests=${resultFilesPath}`, "-Dsonar.language=js", `-Dsonar.working.directory=${getSonarArtifactsDir()}`, `-Dsonar.eslint.reportPaths=${(0, _getFileUtils.getLintReportPath)()}`, "-Dsonar.javascript.lcov.reportPaths=coverage/lcov-report/lcov.info"];
  (0, _fileUtils.checkIfFileExists)(tsConfigurationPath) && command.push(`-Dsonar.typescript.tsconfigPath=${tsConfigurationPath}`);
  return command;
}

/**
 * @function getProjectNameFrmGit - get project from git
 * @returns {string}
 */

function getProjectNameFrmGit(developerName) {
  let $repoName = getRepoName();
  let $branchName = getBranchName();
  let $username = getGitUsername(developerName);
  return $repoName && $branchName && $username ? `${$repoName}_${$branchName}_${$username}` : "desk_default_projects";
}

/**
 * @function getGitUsername - get username from git
 * @returns {string}
 */

function getGitUsername(developerName) {
  try {
    if (developerName) {
      return developerName.split("@")[0];
    }
    const userName = getSpawnOutput("git", ["config", "user.email"]);
    return userName.pop().split("@")[0];
  } catch (error) {
    _logger.Logger.log(_logger.Logger.INFO_TYPE, `make sure that you have propery setup git`);
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
    process.exit(0);
  }
}

/**
 * @function getRepoName - get repo name from git
 * @returns {string}
 */

function getRepoName() {
  try {
    const repoName = getSpawnOutput("git", ["remote", "get-url", "origin"]);
    return repoName.pop().split("/").pop().split(".")[0];
  } catch (error) {
    _logger.Logger.log(_logger.Logger.INFO_TYPE, `make sure that you have propery setup git`);
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
    process.exit(0);
  }
}

/**
 * @function getBranchName - get branch name from git
 * @returns {string}
 */

function getBranchName() {
  // git rev-parse --abbrev-ref HEAD
  try {
    var branchName = getSpawnOutput("git", ["rev-parse", "--abbrev-ref", "HEAD"]); // to get current branch
    return branchName.shift().trim();
  } catch (error) {
    _logger.Logger.log(_logger.Logger.INFO_TYPE, `make sure that you have propery setup git`);
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
    process.exit(0);
  }
}

/**
 * @function getSpawnOutput -  run execution in spawn method
 * @param {string} command - prefix command
 * @param {Array<string>} params - command 
 * @returns {string | any}
 */

function getSpawnOutput(command, params = []) {
  const demon_process = (0, _child_process.spawnSync)(command, params, {
    shell: true
  });
  return demon_process.stdout.toString().split("\n").filter(Boolean);
}

/**
 * @function getCredentials -  get credentials for sonarQube
 * @returns {Object}
 */

function getCredentials() {
  const {
    metric_token
  } = (0, _fileUtils.getLintConfiguration)();
  return (0, _hash.decrypt)(metric_token, 7);
}

/**
 * @function getSonarArtifactsDir - get artifacts path for sonarQube
 * @returns {string}
 */

function getSonarArtifactsDir() {
  return _path.default.resolve(process.cwd(), "lint-report", ".scannerwork");
}
function getTsConfigurationPath() {
  const {
    tsConfigurationPath = undefined
  } = (0, _fileUtils.getLintConfiguration)();
  return tsConfigurationPath && (0, _fileUtils.checkIfFileExists)(tsConfigurationPath) ? tsConfigurationPath : undefined;
}