"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _fs = _interopRequireWildcard(require("fs"));
var _path = _interopRequireDefault(require("path"));
var _logger = require("./logger");
var _child_process = require("child_process");
var _cliArgsToObject = require("./cliArgsToObject");
var _PathUtilities = _interopRequireDefault(require("./PathUtilities"));
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
class FileUtilities {
  /**
  * @function checkIfFileExists - checks if a file exists.
  * @param {string}  - the path to the file.
  * @returns {boolean}
  */

  static checkIfFileExists(file) {
    try {
      _fs.default.accessSync(file, _fs.default.constants.F_OK);
      return true;
    } catch (err) {
      return false;
    }
  }

  /**
   * @function readFileContents -  reads the contents of a file.
   * @param {string}  - the path to the file.
   * @returns {string|any}
   */

  static readFileContents(filePath) {
    try {
      let fileContents = _fs.default.readFileSync(filePath, "utf-8");
      return fileContents;
    } catch (err) {
      return null;
    }
  }

  /**
   * @function writeFileContents -  writes content to a file, creating directories if necessary.
   * @param {string}  - the path to the file.
   * @param {string}  - the content to write to the file.
   * @param {Object} [writeOptions={}] - optional write options.
   * @throws - will throw an error if the write operation fails.
   * @returns {void}
   */

  static writeFileContents(filePath, content, writeOptions = {}) {
    const directoryPath = _path.default.dirname(filePath);
    // Check if the directory exists
    if (!_fs.default.existsSync(directoryPath)) {
      _fs.default.mkdirSync(directoryPath, {
        recursive: true
      });
    }
    try {
      _fs.default.writeFileSync(`${filePath}`, content, writeOptions);
    } catch (err) {
      throw new Error(err);
    }
  }

  /**
   * @function deleteFile - deletes a file if it exists.
   * @param {string}  - The path to the file.
   * @throws - will throw an error if the delete operation fails.
   */

  static deleteFile(filePath) {
    if (checkIfFileExists(filePath)) {
      try {
        _fs.default.unlinkSync(filePath);
      } catch (err) {
        throw new Error(`Error while deleting the test data file: ${filePath}`);
      }
    }
  }

  /**
   * @function deleteFolder - deletes a folder and its contents if it exists.
   * @param {string}  - The path to the folder.
   * @throws Will throw an error if the delete operation fails.
   */

  static deleteFolder(folderPath) {
    if (_fs.default.existsSync(folderPath)) {
      try {
        _fs.default.rmdirSync(folderPath, {
          recursive: true
        });
      } catch (err) {
        throw new Error(`Error while deleting the test data file: ${folderPath}`);
      }
    }
  }

  /**
   * @function getFilesPathFromRootDir - recursively gets the paths of all files in a directory.
   * @param {string}  - The path to the directory.
   * @param {Array<string>}  - An array to hold the file paths.
   * @returns {Array<string>}
   */

  static getFilesPathFromRootDir(directoryPath, files = []) {
    const absoluteDirPath = _path.default.resolve(directoryPath);
    if (_fs.default.statSync(absoluteDirPath).isFile()) {
      return filterFiles([absoluteDirPath]);
    }
    const fileList = _fs.default.readdirSync(absoluteDirPath);
    fileList.forEach(file => {
      file = _path.default.resolve(absoluteDirPath, file);
      if (_fs.default.statSync(file).isDirectory()) {
        this.getFilesPathFromRootDir(file, files);
      }
      if (_fs.default.statSync(file).isFile()) {
        files.push(file);
      }
    });
    return filterFiles(files);
  }

  /**
   * @function getLintConfiguration - gets the path to the lint properties file.
   * @returns {string}
   */

  static getLintConfiguration() {
    const actualLintConfPath = _path.default.resolve(process.cwd(), "lint.config.js");
    const defaultLintConfPath = _path.default.resolve(__dirname, "..", "setup", "sampleLint.config.js");
    return this.checkIfFileExists(actualLintConfPath) ? require(actualLintConfPath) : require(defaultLintConfPath);
  }
  /**
   * @function {isDirectory} - function checks given path is file or directory
   * @param {string} path - path of file to check
   * @returns {Boolean}
   */

  static isDirectory(path) {
    return _fs.default.statSync(path).isDirectory();
  }

  /**
   * @function checkConfigurationFileExist - check configuration present or not and console warn
   * @returns {void}
   */

  static checkConfigurationFileExist() {
    const isConfigPresent = (0, _fs.existsSync)(_path.default.resolve(process.cwd(), "lint.config.js"));
    isConfigPresent ? null : _logger.Logger.log(_logger.Logger.INFO_TYPE, `\n configuration file doesn't present kindly following command - npx ZDLintingFramework init`);
    return isConfigPresent;
  }

  /**
  * @function getFilesFromBranchDiff  - filter file path From git diff json 
  * @param {JSON} diffJson -  Git Diff JSON  (This JSON get by git diff api)
  * @returns {Array<String>} 
  */

  static getFilesFromBranchDiff(diffJson) {
    const rootPath = _PathUtilities.default.getRootGitPath();
    const diffFiles = diffJson === null || diffJson === void 0 ? void 0 : diffJson.diffs.map(diff => {
      try {
        const pathOfFile = _path.default.resolve(rootPath, diff === null || diff === void 0 ? void 0 : diff.new_path);
        if (this.checkIfFileExists(pathOfFile)) {
          return pathOfFile;
        }
        return undefined;
      } catch (error) {
        _logger.Logger.log(_logger.Logger.FAILURE_TYPE, ` file does not exist. make sure that branch upto date ...`);
      }
    }).filter(Boolean);
    return filterFiles(diffFiles);
  }

  /**
   * @function {getStagedFiles_preCommit} - methods get staged files
   * @returns {Array<String>}
   */

  static getStagedFiles_preCommit() {
    const command = "git diff --cached -U0 HEAD --name-only";
    const child = (0, _child_process.spawnSync)(command, {
      shell: true
    });
    const diffFiles = child.stdout.toString().split("\n").map(filePath => _path.default.resolve(_PathUtilities.default.getRootGitPath(), filePath));
    diffFiles.pop(); // remove last empty string
    return diffFiles;
  }

  /**
   * @function getStagedFiles_CI - check Linter trigger from and Filter lint files
   * @param {JSON} branch_diff_json - Git Diff JSON  (This JSON get by git diff api)
   * @returns {Array<String>}
   */

  static getStagedFiles_CI(branch_diff_json) {
    if (typeof branch_diff_json === "boolean" && !branch_diff_json) {
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `Expected Either branch diff json or files. but received ${typeof branch_diff_json === "boolean" ? `empty string` : "empty diff"} `);
      process.exit(0);
    }
    return this.getFilesFromBranchDiff(branch_diff_json);
  }

  /**
   * 
   * @function getFiles - method returns a js and feature and branchDiff based on env
   * @param {string} env - gives running env
   * @param {string} dir - path of directory or file
   * @param {path} branchDiffPath - path of branch diff json
   * @returns {{JsFiles:[],cssFiles:[],featureFiles:[],branch_diff_json:[]}}
   */

  static getFiles(env, dir, branchDiffPath = false) {
    if (env == 'dev') {
      const multipleFiles = dir.split(',');
      const filesAfterFilter = multipleFiles.map(_dir => {
        const listOfFiles = this.checkIfFileExists(_dir) ? _dir : false;
        if (!listOfFiles && typeof listOfFiles === 'boolean') {
          _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `Given file or directory does not exist ...`);
          process.exit(0);
        }
        return this.getFilesPathFromRootDir(listOfFiles, []);
      });
      return (0, _cliArgsToObject.mergeArrayObjects)(filesAfterFilter);
    }
    if (env == 'ci') {
      const _branch_diff_json = branchDiffPath && this.checkIfFileExists(branchDiffPath) ? require(_PathUtilities.default.resolvePath(branchDiffPath)) : false;
      if (!_branch_diff_json) {
        _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `branch diff file does not exists ...`);
        process.exit(0);
      }
      return {
        ...this.getStagedFiles_CI(_branch_diff_json),
        branch_diff_json: _branch_diff_json
      };
    }
  }

  /**
   * @function filterFiles - filter javascript files. omit feature files
   * @param {Array<String>} lintFiles - linter files as Array
   * @returns {Array<String>} 
   */

  static filterFiles(lintFiles) {
    return lintFiles.reduce((files, currentFile) => {
      if (currentFile.includes('.feature') && this.checkIfFileExists(currentFile)) {
        files.featureFiles.push(currentFile);
      }
      if (currentFile.includes('.js') || currentFile.includes('.ts') || currentFile.includes('.tsx') || currentFile.includes('.jsx') && this.checkIfFileExists(currentFile)) {
        files.JsFiles.push(currentFile);
      }
      if (currentFile.includes('.css') && this.checkIfFileExists(currentFile)) {
        files.CssFiles.push(currentFile);
      }
      return files;
    }, {
      featureFiles: [],
      JsFiles: [],
      CssFiles: []
    });
  }

  /**
   * @function {removeFile} - this methods remove diff json file
   * @returns {void}
   */

  static removeFile(file_path) {
    this.checkIfFileExists(file_path) ? (0, _fs.unlinkSync)(file_path) : null;
  }

  /**
  * @function {filterExemptionFiles} - method filter exemption need files
  * @param {Array>String>} exemptionFiles - exemption needed files
  * @param {Array>String>} actualFiles - actualFiles from git diff
  * @returns {Array>String>}
  */

  static filterExemptionFiles(exemptionFiles, actualFiles) {
    if (exemptionFiles.length == 0) {
      return actualFiles;
    }
    return actualFiles.filter(filePath => {
      const relativePath = _path.default.relative(process.cwd(), filePath);
      return !exemptionFiles.includes(relativePath);
      // if(exemptionFiles.includes(file)){
      //   return false
      // }
      // return true
    });
  }
}
var _default = exports.default = FileUtilities;