"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkConfigurationFileExist = checkConfigurationFileExist;
exports.checkIfFileExists = checkIfFileExists;
exports.deleteFile = deleteFile;
exports.deleteFolder = deleteFolder;
exports.getFilesPathFromRootDir = getFilesPathFromRootDir;
exports.getLintConfiguration = getLintConfiguration;
exports.isDirectory = isDirectory;
exports.readFileContents = readFileContents;
exports.writeFileContents = writeFileContents;
var _fs = _interopRequireWildcard(require("fs"));
var _path = _interopRequireDefault(require("path"));
var _getFileUtils = require("../core/eslint/getFileUtils");
var _logger = require("./logger");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
/**
 * @function checkIfFileExists - checks if a file exists.
 * @param {string}  - the path to the file.
 * @returns {boolean}
 */

function checkIfFileExists(file) {
  try {
    _fs.default.accessSync(file, _fs.default.constants.F_OK);
    return true;
  } catch (err) {
    return false;
  }
}

/**
 * @function readFileContents -  reads the contents of a file.
 * @param {string}  - the path to the file.
 * @returns {string|any}
 */

function readFileContents(filePath) {
  try {
    let fileContents = _fs.default.readFileSync(filePath, "utf-8");
    return fileContents;
  } catch (err) {
    return null;
  }
}

/**
 * @function writeFileContents -  writes content to a file, creating directories if necessary.
 * @param {string}  - the path to the file.
 * @param {string}  - the content to write to the file.
 * @param {Object} [writeOptions={}] - optional write options.
 * @throws - will throw an error if the write operation fails.
 * @returns {void}
 */

function writeFileContents(filePath, content, writeOptions = {}) {
  const directoryPath = _path.default.dirname(filePath);
  // Check if the directory exists
  if (!_fs.default.existsSync(directoryPath)) {
    _fs.default.mkdirSync(directoryPath, {
      recursive: true
    });
  }
  try {
    _fs.default.writeFileSync(`${filePath}`, content, writeOptions);
  } catch (err) {
    throw new Error(err);
  }
}

/**
 * @function deleteFile - deletes a file if it exists.
 * @param {string}  - The path to the file.
 * @throws - will throw an error if the delete operation fails.
 */

function deleteFile(filePath) {
  if (checkIfFileExists(filePath)) {
    try {
      _fs.default.unlinkSync(filePath);
    } catch (err) {
      throw new Error(`Error while deleting the test data file: ${filePath}`);
    }
  }
}

/**
 * @function deleteFolder - deletes a folder and its contents if it exists.
 * @param {string}  - The path to the folder.
 * @throws Will throw an error if the delete operation fails.
 */

function deleteFolder(folderPath) {
  if (_fs.default.existsSync(folderPath)) {
    try {
      _fs.default.rmdirSync(folderPath, {
        recursive: true
      });
    } catch (err) {
      throw new Error(`Error while deleting the test data file: ${folderPath}`);
    }
  }
}

/**
 * @function getFilesPathFromRootDir - recursively gets the paths of all files in a directory.
 * @param {string}  - The path to the directory.
 * @param {Array<string>}  - An array to hold the file paths.
 * @returns {Array<string>}
 */

function getFilesPathFromRootDir(directoryPath, files = []) {
  const absoluteDirPath = _path.default.resolve(directoryPath);
  if (_fs.default.statSync(absoluteDirPath).isFile()) {
    return (0, _getFileUtils.filterFiles)([absoluteDirPath]);
  }
  const fileList = _fs.default.readdirSync(absoluteDirPath);
  fileList.forEach(file => {
    file = _path.default.resolve(absoluteDirPath, file);
    if (_fs.default.statSync(file).isDirectory()) {
      getFilesPathFromRootDir(file, files);
    }
    if (_fs.default.statSync(file).isFile()) {
      files.push(file);
    }
  });
  return (0, _getFileUtils.filterFiles)(files);
}

/**
 * @function getLintConfiguration - gets the path to the lint properties file.
 * @returns {string}
 */

function getLintConfiguration() {
  const actualLintConfPath = _path.default.resolve(process.cwd(), "lint.config.js");
  const defaultLintConfPath = _path.default.resolve(__dirname, "..", "setup", "sampleLint.config.js");
  return checkIfFileExists(actualLintConfPath) ? require(actualLintConfPath) : require(defaultLintConfPath);
}
/**
 * @function {isDirectory} - function checks given path is file or directory
 * @param {string} path - path of file to check
 * @returns {Boolean}
 */

function isDirectory(path) {
  return _fs.default.statSync(path).isDirectory();
}

/**
 * @function checkConfigurationFileExist - check configuration present or not and console warn
 * @returns {void}
 */

function checkConfigurationFileExist() {
  const isConfigPresent = (0, _fs.existsSync)(_path.default.resolve(process.cwd(), "lint.config.js"));
  isConfigPresent ? null : _logger.Logger.log(_logger.Logger.INFO_TYPE, `\n configuration file doesn't present kindly following command - npx ZDLintingFramework init`);
  return isConfigPresent;
}